@@ -1,4 +1,4 @@
-use std::collections::{BTreeMap, BTreeSet, HashMap, HashSet};
+use std::collections::{BTreeMap, BTreeSet, HashMap};
 use std::convert::TryFrom;
 use std::fs::File;
 use std::io::Read;
@@ -7,7 +7,7 @@ use std::path::Path;
 use std::string::ToString;
 use std::sync::Mutex;
 
-use anyhow::Result;
+use anyhow::{Context, Result};
 use serde_yaml;
 use vec_map::VecMap;
 
@@ -146,7 +146,24 @@ impl Scenario {
         let mut scenario_content = String::new();
         File::open(path)?.read_to_string(&mut scenario_content)?;
 
-        Ok(serde_yaml::from_str(&scenario_content)?)
+        let mut scenario: Self = serde_yaml::from_str(&scenario_content)?;
+
+        let mut event_expressions = HashMap::new();
+
+        // register all events as expressions
+        for (name, formula) in scenario.events() {
+            let identifier = ExpressionIdentifier(name.clone());
+            if !scenario.expressions().contains_key(&identifier) {
+                event_expressions.insert(identifier, formula.clone());
+            }
+        }
+        let absent_identifier = ExpressionIdentifier("absent".to_owned());
+        if !scenario.expressions.contains_key(&absent_identifier) {
+            event_expressions.insert(absent_identifier, Formula::absent(&scenario));
+        }
+        scenario.expressions.extend(event_expressions);
+
+        Ok(scenario)
     }
 
     pub(crate) fn sample_info<T>(&self) -> SampleInfoBuilder<T> {
@@ -178,10 +195,14 @@ impl Scenario {
     }
 
     pub(crate) fn vaftrees(&self, contig: &str) -> Result<HashMap<String, VAFTree>> {
+        println!("Preprocessing events for contig {}", contig);
         self.events()
             .iter()
             .map(|(name, formula)| {
-                let normalized = formula.normalize(self, contig)?;
+                let normalized = formula
+                    .normalize(self, contig)
+                    .with_context(|| format!("invalid event definition for {}", name))?;
+                println!("    {}: {}", name, normalized);
                 let vaftree = VAFTree::new(&normalized, self, contig)?;
                 Ok((name.to_owned(), vaftree))
             })
