@@ -5,15 +5,17 @@ use bio::stats::probs::LogProb;
 use itertools::Itertools;
 use strum::IntoEnumIterator;
 
-use crate::utils::{PROB_05, PROB_095};
+use crate::utils::PROB_095;
 use crate::variants::evidence::observation::{Observation, ReadPosition};
 
 pub(crate) mod read_orientation_bias;
 pub(crate) mod read_position_bias;
+pub(crate) mod softclip_bias;
 pub(crate) mod strand_bias;
 
 pub(crate) use read_orientation_bias::ReadOrientationBias;
 pub(crate) use read_position_bias::ReadPositionBias;
+pub(crate) use softclip_bias::SoftclipBias;
 pub(crate) use strand_bias::StrandBias;
 
 pub(crate) trait Bias: Default + cmp::PartialEq {
@@ -31,7 +33,7 @@ pub(crate) trait Bias: Default + cmp::PartialEq {
         })
     }
 
-    fn is_informative(&self, pileups: &[Vec<Observation<ReadPosition>>]) -> bool {
+    fn is_informative(&self, _pileups: &[Vec<Observation<ReadPosition>>]) -> bool {
         true
     }
 
@@ -71,6 +73,8 @@ pub(crate) struct Biases {
     read_orientation_bias: ReadOrientationBias,
     #[getset(get = "pub(crate)")]
     read_position_bias: ReadPositionBias,
+    #[getset(get = "pub(crate)")]
+    softclip_bias: SoftclipBias,
 }
 
 impl Biases {
@@ -78,8 +82,12 @@ impl Biases {
         consider_read_orientation_bias: bool,
         consider_strand_bias: bool,
         consider_read_position_bias: bool,
+        consider_softclip_bias: bool,
     ) -> Box<dyn Iterator<Item = Self>> {
-        if !consider_strand_bias && !consider_read_orientation_bias && !consider_read_position_bias
+        if !consider_strand_bias
+            && !consider_read_orientation_bias
+            && !consider_read_position_bias
+            && !consider_softclip_bias
         {
             return Box::new(std::iter::empty());
         }
@@ -99,23 +107,41 @@ impl Biases {
         } else {
             vec![ReadOrientationBias::None]
         };
+        let softclip_biases = if consider_softclip_bias {
+            SoftclipBias::iter().collect_vec()
+        } else {
+            vec![SoftclipBias::None]
+        };
 
         Box::new(
             strand_biases
                 .into_iter()
                 .cartesian_product(read_orientation_biases.into_iter())
                 .cartesian_product(read_position_biases.into_iter())
-                .filter_map(|((sb, rob), rpb)| {
-                    match (sb.is_artifact(), rob.is_artifact(), rpb.is_artifact()) {
-                        (true, false, false) | (false, true, false) | (false, false, true) => Some(
+                .cartesian_product(softclip_biases.into_iter())
+                .filter_map(|(((sb, rob), rpb), scb)| {
+                    if [
+                        sb.is_artifact(),
+                        rob.is_artifact(),
+                        rpb.is_artifact(),
+                        scb.is_artifact(),
+                    ]
+                    .into_iter()
+                    .map(|artifact| if *artifact { 1 } else { 0 })
+                    .sum::<usize>()
+                        == 1
+                    {
+                        Some(
                             BiasesBuilder::default()
                                 .strand_bias(sb)
                                 .read_orientation_bias(rob)
                                 .read_position_bias(rpb)
+                                .softclip_bias(scb)
                                 .build()
                                 .unwrap(),
-                        ),
-                        _ => None,
+                        )
+                    } else {
+                        None
                     }
                 }),
         )
@@ -126,6 +152,7 @@ impl Biases {
             .strand_bias(StrandBias::None)
             .read_orientation_bias(ReadOrientationBias::None)
             .read_position_bias(ReadPositionBias::None)
+            .softclip_bias(SoftclipBias::None)
             .build()
             .unwrap()
     }
@@ -134,18 +161,21 @@ impl Biases {
         self.strand_bias.is_possible(pileups)
             && self.read_orientation_bias.is_possible(pileups)
             && self.read_position_bias.is_possible(pileups)
+            && self.softclip_bias.is_possible(pileups)
     }
 
     pub(crate) fn is_informative(&self, pileups: &[Vec<Observation<ReadPosition>>]) -> bool {
         self.strand_bias.is_informative(pileups)
             && self.read_orientation_bias.is_informative(pileups)
             && self.read_position_bias.is_informative(pileups)
+            && self.softclip_bias.is_informative(pileups)
     }
 
     pub(crate) fn is_likely(&self, pileups: &[Vec<Observation<ReadPosition>>]) -> bool {
         self.strand_bias.is_likely(pileups)
             && self.read_orientation_bias.is_likely(pileups)
             && self.read_position_bias.is_likely(pileups)
+            && self.softclip_bias.is_likely(pileups)
     }
 
     pub(crate) fn prob(&self, observation: &Observation<ReadPosition>) -> LogProb {
@@ -153,17 +183,20 @@ impl Biases {
         self.strand_bias.prob(observation)
             + self.read_orientation_bias.prob(observation)
             + self.read_position_bias.prob(observation)
+            + self.softclip_bias.prob(observation)
     }
 
     pub(crate) fn prob_any(&self, observation: &Observation<ReadPosition>) -> LogProb {
         self.strand_bias.prob_any(observation)
             + self.read_orientation_bias.prob_any(observation)
             + self.read_position_bias.prob_any(observation)
+            + self.softclip_bias.prob_any(observation)
     }
 
     pub(crate) fn is_artifact(&self) -> bool {
         self.strand_bias.is_artifact()
             || self.read_orientation_bias.is_artifact()
             || self.read_position_bias.is_artifact()
+            || self.softclip_bias.is_artifact()
     }
 }
